#include <string>
#include <iostream>
#include <vector>

#include "iopclib/iopcHeaders.h"
#include "school.h"
#include "testClass.h"
#include "iopcdb/sqlStatementsFeature.h"
#include "iopclib/cache/voidCache.h"
#include "iopclib/cache/arcCacheST.h"
#include "iopclib/cache/lruCacheST.h"

//using namespace iopc;
using namespace std;
using namespace iopc;


void setupDB(Connection* conn)
{
	vector<string> script;

	// drop script
	script = ScriptsGenerator::getDbDropScript(conn->getDriver());

	for(vector<string>::const_iterator it = script.begin(); it != script.end(); it++) {
		//LogWriter::logInfo(*it);
		try {
			conn->sqlNonQuery(*it);
		} catch (...) {}
	}

	// create script
	script = ScriptsGenerator::getDbCreateScript(conn->getDriver());

	for(vector<string>::const_iterator it = script.begin(); it != script.end(); it++) {
		conn->sqlNonQuery(*it);
		//LogWriter::logInfo(*it);
	}
}

void dbTest(Connection* conn)
{
	/*Driver& d = conn->getDriver();
	if (d.supportsFeature<SqlStatementsFeature>()) {
		TypeMappingFeature& f = d.getFeature<TypeMappingFeature>();
		string sqlType = f.getTypeSql(TypeDesc<string>::getType());
		cout << sqlType << endl;
	}*/

	conn->sqlNonQuery("insert into blacklist values(1, 'Richard Doe')");
	conn->sqlNonQuery("insert into blacklist values(2, 'Ola Nordmann')");
	Cursor* cur = conn->sql("select id, name from blacklist");
	EString name;
	EInt id;
	MetadataHolder meta;
	cur->execute();
	cur->addEnhancedOutParam(1, id);
	cur->addEnhancedOutParam(2, name, meta);
	while (cur->fetchNext()) {
		cout << id << " " << name << endl;
	}
	cur->close();
	cur->open();
	cur->execute();
	while (cur->fetchNext()) {
		cout << id << " " << name << endl;
	}
	cur->close();
	conn->returnCursor(cur);
	conn->commit();
}



int main(int argc, char *argv[])
{
	/*const Type& tOidObject = TypeDesc<OidObject>::getType();
	tOidObject["db.table"].setStringValue("OOOidObject");
	tOidObject["db.type"].setStringValue("tOOOidObject");
	tOidObject["db.mapping.type"].setStringValue("object");
	const Type& tTestClass = TypeDesc<TestClass>::getType();
	tTestClass["db.mapping.type"].setStringValue("object");
	const Type& tTestClass2 = TypeDesc<TestClass2>::getType();
	tTestClass2["db.mapping.type"].setStringValue("object");
	const Type& tTestClass3 = TypeDesc<TestClass3>::getType();
	tTestClass3["db.mapping.type"].setStringValue("object");*/

	try {
		//MetadataHolder::getDefaultsMeta()["db.mapping.type"].setStringValue("object");
		Iopc::init(new TextFileMetadataLoader("metadata.config"), LogWriter::WARNING | LogWriter::ERROR | LogWriter::INFO_SQL /*| LogWriter::DEBUG | LogWriter::TRACE | LogWriter::INFO*/);
		//LogWriter::getInstance().traceMaskTTY = ( );
		cout << "Start" << endl;

		cout << "Registered types" << endl;
		const map<string, const Type*>& types = TypeManager::getTypesMap();
		for(map<string, const Type*>::const_iterator it = types.begin(); it != types.end(); it++) {
			cout << (*it).first << endl;
		}

		cout << "Description of the Person type" << endl;
		const Type& t = TypeDesc<Person>::getType();
		cout << "Type name: " << t.getQualifiedName() << endl;
		cout << "Attributes: " << endl;
		const Type::AttributesMap& attributes = t.getAttributes();
		for(Type::AttributesIterator it = attributes.begin(); it != attributes.end(); it++) {
			const Attribute& attr = it->second;
			switch (attr.getVisibility()) {
				case Attribute::VISIBILITY_PUBLIC: cout << "public "; break;
				case Attribute::VISIBILITY_PRIVATE: cout << "private "; break;
				case Attribute::VISIBILITY_PROTECTED: cout << "protected "; break;
			}
			cout << it->second.getType().getQualifiedName()  << " " << it->first << endl;
		}
		cout << "Inherited attributes: " << endl;
		const Type::InheritedAttributesMap& inheritedAttributes = t.getInheritedAttributes();
		for(Type::InheritedAttributesIterator it = inheritedAttributes.begin(); it != inheritedAttributes.end(); it++) {
			cout << "  " << it->first << ": " << it->second->getType().getQualifiedName() << endl;
		}
		cout << "Parents: " << endl;
		const Type::ParentsList& parents = t.getParents();
		for(Type::ParentsIterator it = parents.begin(); it != parents.end(); it++) {
			cout << " " << (*it)->getQualifiedName() << endl;
		}
		cout << "Children: " << endl;
		const Type::ChildrenList& children = t.getChildren();
		for(Type::ChildrenIterator it = children.begin(); it != children.end(); it++) {
			cout << "  " << (*it)->getQualifiedName() << endl;
		}
		// new instance
		Employee* e = (Employee*) TypeManager::getType("::Employee").newInstance();
		*((EString*)e->getAttributeValue("name").getValue()) = "Ola Nordmann";
		*((EShort*)e->getAttributeValue("age").getValue()) = 45;
		*((EInt*)e->getAttributeValue("salary").getValue()) = 60000;
		cout << e->name << " " << e->age << " " << e->salary << endl;

		const Object::AttributeValuesMap& attrVals = e->getAttributeValues();
		for(Object::AttributeValuesIterator it = attrVals.begin(); it != attrVals.end(); it++) {
			const AttributeValue& val = it->second;
			cout << it->first << ": ";
			if (val.getAttribute().getType().getDataTypeClass() == Type::IOPC_TYPECLASS_ENHANCED) {
				cout << ((EnhancedTypeBase*)val.getValue())->toString() << endl;
			}
		}

		const Type& t1 = TypeDesc<Person>::getType();
		const Type& t2 = TypeDesc<Student>::getType();
		if (t1 == t2) { cout << "t1 is t2" << endl; }
		if (t1 != t2) { cout << "t1 is not t2" << endl; }
		if (t1.isTypeOf(t2)) { cout << "t1 is t2 or t1 is descendant of t2" << endl; }
		if (t2.isTypeOf(t1)) { cout << "t1 is t2 or t2 is descendant of t1" << endl; }
		if (t1.is<Student>()) { cout << "t1 is of type Student" << endl; }
		if (t2.is<Student>()) { cout << "t2 is of type Student" << endl; }
		if (t1.isTypeOf<Person>()) { cout << "t1 is of type Person or its descendant" << endl; }
		if (t2.isTypeOf<Person>()) { cout << "t2 is of type Person or its descendant" << endl; }







		delete e;

		// Enhanced datatypes

		EInt a = 8;
		a = a + 2;
		int b = 2 + a;
		EFloat d = 2.5;
		d = (d + 2.5 + a) / 2;
		EDouble c = (a + 2)*(a + 3);
		cout << "a: " << a << " b: " << b << " c: " << c << " d: " << d << endl;
		d.setNull();
		cout << "a.isDirty(): " << a.isDirty() << endl;
		cout << "c.isDirty(): " << c.isDirty() << endl;
		cout << "d: " << d.toString() << endl;
		//" b " << b << " c: " << c << " d: " << d << endl;



		Database* db = NULL;
		CachedConnection* conn = NULL;

		try {
			//db = new CachedDatabase(DriverManager::getDriver("IopcOracle10g").getDatabase("XE"), new ComposedCache());
			//VoidCache* v = new VoidCache();
			db = new CachedDatabase(DriverManager::getDriver("IopcOracle10g").getDatabase("XE"), new VoidCache());
			conn = (CachedConnection*)db->getConnection("username=iopc;password=iopc;autocommit=false");
/*			CacheKeeper cacheKeeper(-15000, 20, 40);
			cacheKeeper.setCache(conn, new HashLruCache());
			cacheKeeper.setCache(conn, TypeDesc<Employee>::getType(), new HashArcCache());
			cacheKeeper.setStrategy(conn, Strategy::lazy);
			cacheKeeper.runMaintainer();*/
			conn->open();


			setupDB(conn);
			dbTest(conn);


			DbPtr<TestClass> test1;
			test1->attrInt = 3;
			test1->attrString = "hello";
			test1.bePersistent(conn);
			int oid = test1->oid;
			conn->commit();
			conn->removeAll();
			DbPtr<TestClass> test2(conn, oid);
			cout << test2->attrString << endl;
			test2->attrString = "hello2";
			conn->commit();


			const Type& t = TypeDesc<TestClass>::getType();
			cout << t.getQualifiedName() << endl;

			DbPtr<Person> person;
			person->name = "Mary Major";
			person->age = 60;
			person.bePersistent(conn);

			DbPtr<Employee> employee;
			employee->name = "Ola Nordmann";
			employee->age = 45;
			employee->salary = 60000;
			employee.bePersistent(conn);
			//conn->commit();
			//cout << employee->oid << endl;

			DbPtr<Student> student;
			student->name = "Richard Doe";
			student->age = 22;
			student->studcardid = "WCD-3223";
			//student->supervisor = employee;
			student.bePersistent(conn);;

			DbPtr<PhdStudent> phd;
			phd->name = "Joe Bloggs";
			phd->age = 27;
			phd->studcardid = "PHD-1234";
			phd->scholarship = 12000;
			phd.bePersistent(conn);

			{
				CachePtr<PhdStudent> phdPtr = *phd;
				phdPtr->scholarship = 13000;
				//phdPtr->supervisor = employee;
				//phdPtr.release();
			}

			DbPtr<PhdStudent> phd2 = phd;
			//phd.dbDelete();
			phd->scholarship = 14000;

			conn->commit();
			phd.dbDelete();
			conn->commit();
			KeyValues key;
			key.add("externalId", 2);
			DbPtr<BlacklistEntry> entry(conn, key);
			cout << entry->externalId << " " << entry->name << endl;
			DbPtr<BlacklistEntry> newEntry;
			newEntry->externalId = 3;
			newEntry->name = "Mary Major";
			newEntry.bePersistent(conn);


			//SimpleQuery q("$::UserTable::tablespaceName$ = 'USERS'", "$::UserTable::tableName$ ASC");
			FreeQuery q("USER_TABLES JOIN USER_TAB_COLS USING (TABLE_NAME) GROUP BY TABLESPACE_NAME, TABLE_NAME");
			Result<UserTable> userTables(conn, q);
			userTables.open();
			for (Result<UserTable>::iterator it = userTables.begin(); it != userTables.end(); it++) {
				cout << it->tableName << " " << it->tablespaceName << " " << it->columnCount << endl;
			}
			userTables.close();


			//s1->setData("Richard Doe", 22);
			/*s1->name = "Richard Doe";
			s1->age = 22;
			s1->studcardid = "WCD-3223";
			s1.bePersistent(conn);

			//s2->setData("Joe Bloggs", 27);
			s2->name = "Joe Bloggs";
			s2->age = 27;
			s2->studcardid = "PHD-1234";
			s2->scholarship = 12000;
			s2.bePersistent(conn);

			conn->commit();*/

			/*empl->name = "James Bond";
			//empl->country = "US";
			//empl->idcard = "123456";
			empl->salary = 8000;
			empl.bePersistent(conn);
			stud->name = "Ronin";
			//stud->country = "UK";
			//stud->idcard = "DD446";
			stud->studcardid = "12345-d";
			stud->scholarship = 100;
			stud->supervisor = empl;
			stud.bePersistent(conn);*/


			conn->commit();


			cout << "Persons age >= 30" << endl;
			FreeQuery q2("$::Person$ WHERE $::Person::age$ >= 30");
			Result<Person> r(conn, q2);
			r.open();
			for (Result<Person>::iterator it = r.begin(); it != r.end(); it++) {
				cout << "name: " << it->name << " age: " << it->age << endl;
			}
			r.close();
			//cacheKeeper.stopMaintainer();
			conn->close();
		} catch (IopcException& e) {
			cout << e.what() << endl;
		}

		if (db && conn) db->returnConnection(conn);
		if (db) DriverManager::returnDatabase(db);


		cout << "end" << endl;
		Iopc::shutdown();
	} catch (IopcException& e) {
		cout << e.what();
	}
	cin.getline(NULL, 0);
	return 0;
}


