/**
 * @file iopcdb.cpp
 *
 * @author Petr Cermak
 */

#include "iopcDb.h"
#include "driverManager.h"
#include "iopcmeta/iopcMeta.h"
#include "iopcmeta/enhancedString.h"

namespace iopc {

void IopcDb::init(MetadataInitializer* initializer, int traceMaskTTY, int traceMask, const std::string& logFileName, bool initDependencies)
{
	if (initDependencies) {
		if (initializer != NULL) {
			IopcMeta::init(initializer, traceMaskTTY, traceMask, logFileName);
		} else {
			IopcMeta::init(traceMaskTTY, traceMask, logFileName);
		}
	}

	if (!TypeDesc<EString>::getType().hasMeta("db.type.length")) {
		TypeDesc<EString>::getType()["db.type.length"].setIntValue(2000);
	}
	if (!TypeDesc<EWString>::getType().hasMeta("db.type.length")) {
		TypeDesc<EWString>::getType()["db.type.length"].setIntValue(1000);
	}
	if (!TypeDesc<std::string>::getType().hasMeta("db.type.length")) {
		TypeDesc<std::string>::getType()["db.type.length"].setIntValue(2000);
	}
	if (!TypeDesc<std::wstring>::getType().hasMeta("db.type.length")) {
		TypeDesc<std::wstring>::getType()["db.type.length"].setIntValue(1000);
	}

	DriverManager::getInstance().initDrivers();
}

void IopcDb::init(int traceMaskTTY, int traceMask, const std::string& logFileName, bool initDependencies)
{
	init(NULL, traceMaskTTY, traceMask, logFileName, initDependencies);
}

void IopcDb::shutdown(bool shutdownDependencies)
{
	DriverManager::getInstance().shutdownDrivers();
	if (shutdownDependencies) {
		IopcMeta::shutdown();
	}
}

}

