/**
 * @file oracle10gDatabaseImpl.cpp
 *
 * @author Josef Troch Josef Troch (<a href="mailto:josef.troch@email.cz">josef.troch@email.cz</a>) 2003 - 2004
 * @author Petr Cermak
 */


#include "oracle10gDatabaseImpl.h"
#include "iopccommon/trace.h"
#include "oracle10gOci.h" //OCI functions etc.
#include "oracle10gConnectionImpl.h"
#include <string>
#include "iopccommon/util.h"

using namespace std;

#ifdef _MSC_VER
  #pragma warning (disable : 4800) //'int' : forcing value to bool 'true' or 'false' (performance warning)
#endif

namespace iopc {

//Mutex Oracle10gDatabaseImpl::ociLock;

Oracle10gDatabaseImpl::Oracle10gDatabaseImpl(const string& dbName)
{
	this->dbName = dbName;
	ociInit();
}

Oracle10gDatabaseImpl::~Oracle10gDatabaseImpl()
{
	ociShutdown();
}

void Oracle10gDatabaseImpl::ociShutdown()
{
	OCIHandleFree(ociEnvironment, OCI_HTYPE_ENV);
}


void Oracle10gDatabaseImpl::ociInit()
{
	IOPC_FUNC_INVOKED;

	sword ret = OCIEnvCreate(&ociEnvironment, OCI_DEFAULT | OCI_THREADED, NULL, NULL, NULL, NULL, 0, NULL);

	if (!OCI_OK_EXT(ret)) {
		IOPC_LOG_THROW(IopcDbException("Error calling OCIEnvCreate"));
	}
}

ConnectionImpl* Oracle10gDatabaseImpl::getConnection(const string& connStr)
{
	IOPC_FUNC_INVOKED;
	// TODO: connection/session pooling

	map<string, string> params = StringUtil::createParamMap(connStr);
	if (params["username"] == "" || params["password"] == "") {
		IOPC_LOG_THROW(IopcDbException("Connection string must be of format: 'username=...;password=...[;autocommit=(true|false)]'"));
	}

	if (params.find("autocommit") == params.end()) {
		return new Oracle10gConnectionImpl(
				ociEnvironment,
				params["username"],
				params["password"],
				dbName
				);
	} else {
		bool autoCommit;
		if (params["autocommit"] == "true") {
			autoCommit = true;
		} else if (params["autocommit"] == "false") {
			autoCommit = false;
		} else {
			IOPC_LOG_THROW(IopcDbException("Connection string parameter autocommit has invalid value. Only true or false are allowed."));
		}
		return new Oracle10gConnectionImpl(
				ociEnvironment,
				params["username"],
				params["password"],
				dbName,
				autoCommit
				);

	}
}

void Oracle10gDatabaseImpl::returnConnection(ConnectionImpl* connection)
{
	IOPC_FUNC_INVOKED;
	delete connection;
}



}
