/**
 * @file oracle10gDatabaseImpl.h
 * 
 * @author Josef Troch Josef Troch (<a href="mailto:josef.troch@email.cz">josef.troch@email.cz</a>) 2003-2004
 * @author Petr Cermak
 */

#ifndef ORACLE10GDATABASE_H_
#define ORACLE10GDATABASE_H_

#include "iopcdb/databaseImpl.h"
#include "iopccommon/defsPlatform.h"
#include "iopccommon/mutex.h"
#include <string>
#include "oracle10gOci.h"

// Forward declarations
/*struct OCIEnv;
struct OCIError;*/

namespace iopc {

/**
 * @brief Abstraction of a database that is accessed using Oracle 10g OCI
 *
 * Methods of this class return true if OK, false otherwise (if not explicitly said something different).
 * In case of connection error they throw exception ObjLibException_ConnectionError.
 *
 * @see Oracle10gSqlStatements, Oracle10gConnection, Oracle10gCursor
 */ 
class DLL_EXTERNAL Oracle10gDatabaseImpl : public DatabaseImpl
{

//friend class Oracle10gConnection; //to access handles such as ociEnvironment

public:

	Oracle10gDatabaseImpl(const std::string& dbName);

	virtual ~Oracle10gDatabaseImpl();

	virtual ConnectionImpl* getConnection(const std::string& connStr);
	
	virtual void returnConnection(ConnectionImpl* connection);
	
	inline OCIEnv* getOciEnvironment() { return ociEnvironment; }
	
	//inline static Mutex& getLock() { return ociLock; }

private:

	/**
	 * Pointer to OCI Environment handle (see OCIEnvCreate() in Oracle documentation).
	 * OCI can be used in a multitrheaded environment.
	 */
	OCIEnv* ociEnvironment;	 
		
	std::string dbName;

	//! Set if an error occured in constructor while initializing Oracle 10g OCI. If set, Connect() fails.
	//bool errorInitializingOCI; 
	
	//static Mutex ociLock;
	
	inline const std::string getOCIMsg(
		int ociRetVal,
		const std::string funcName) {
			
		return 	iopc::getOCIMsg(ociEnvironment, NULL, ociRetVal, funcName, true);
	}
	
	void ociInit();
	
	void ociShutdown();
	
};

} // namespace iopc

#endif //__ORACLE10GDATABASE_H__
