#include "oracle10gMappingStatementsFeature.h"
#include <string>
#include "iopcmeta/type.h"
#include "iopccommon/globalSettings.h"
#include <sstream>
#include "iopclib/types/dbType.h"
#include <set>
#include "iopccommon/util.h"

using namespace iopc;
using namespace std;

string Oracle10gMappingStatementsFeature::getSelectTypeStatement(const MetadataHolder& attributes, bool useCache)
{
	if (useCache && cache.hasCachedStatement(cache.selectTypeStatementType)) {
		return cache.getCachedStatement(cache.selectTypeStatementType);
	}
	ostringstream ostr;
	ostr << "SELECT " << TypeDesc<OidObject>::getType().getAttribute("className").getMeta("db.column").getStringValue();
	ostr << " FROM " << TypeDesc<OidObject>::getType().getMeta("db.table").getStringValue();
	ostr << " WHERE " << TypeDesc<OidObject>::getType().getAttribute("oid").getMeta("db.column").getStringValue();
	ostr << " = :v1";
	if (useCache) {
		cache.setCachedStatement(cache.selectTypeStatementType, ostr.str());
	}
	return ostr.str();
}

string Oracle10gMappingStatementsFeature::getSelectSerialIdStatement(const MetadataHolder& attributes, bool useCache)
{
	if (useCache && cache.hasCachedStatement(cache.selectSerialIdStatementType)) {
		return cache.getCachedStatement(cache.selectSerialIdStatementType);
	}
	ostringstream ostr;
	ostr << "SELECT " << TypeDesc<OidObject>::getType().getAttribute("serialId").getMeta("db.column").getStringValue();
	ostr << " FROM " << TypeDesc<OidObject>::getType().getMeta("db.table").getStringValue();
	ostr << " WHERE " << TypeDesc<OidObject>::getType().getAttribute("oid").getMeta("db.column").getStringValue();
	ostr << " = :v1";
	if (useCache) {
		cache.setCachedStatement(cache.selectSerialIdStatementType, ostr.str());
	}
	return ostr.str();
}


string Oracle10gMappingStatementsFeature::getNextOidStatement(const MetadataHolder& attributes, bool useCache)
{
	if (useCache && cache.hasCachedStatement(cache.selectNextOidStatementType)) {
		return cache.getCachedStatement(cache.selectNextOidStatementType);
	}
	ostringstream ostr;
	ostr << "SELECT " << globalSettings.getMeta("db.oracle.oidSequence.name").getStringValue() << ".NEXTVAL INTO :v1 FROM DUAL";
	if (useCache) {
		cache.setCachedStatement(cache.selectNextOidStatementType, ostr.str());
	}
	return ostr.str();
}

string Oracle10gMappingStatementsFeature::getNextSerialIdStatement(const MetadataHolder& attributes, bool useCache)
{
	if (useCache && cache.hasCachedStatement(cache.selectNextSerialIdStatementType)) {
		return cache.getCachedStatement(cache.selectNextSerialIdStatementType);
	}
	ostringstream ostr;
	ostr << "SELECT " << globalSettings.getMeta("db.oracle.serialIdSequence.name").getStringValue() << ".NEXTVAL INTO :v1 FROM DUAL";
	if (useCache) {
		cache.setCachedStatement(cache.selectNextSerialIdStatementType, ostr.str());
	}
	return ostr.str();
}

vector<string> Oracle10gMappingStatementsFeature::dbCreateScriptStart(const MetadataHolder& attributes)
{
	vector<string> ret;
	ostringstream str;
	if (!attributes.hasMeta("db.oracle.oidSequence.sql.create")) {
		str << "CREATE SEQUENCE " << globalSettings.getMeta("db.oracle.oidSequence.name").getStringValue() << " MINVALUE 1 START WITH 1 INCREMENT BY 1 CACHE 20 NOCYCLE";
	} else {
		str << attributes.getMeta("db.oracle.oidSequence.sql.create").getStringValue() << endl;
	}
	ret.push_back(str.str());
	ostringstream str2;
	if (!attributes.hasMeta("db.oracle.serialIdSequence.sql.create")) {
		str2 << "CREATE SEQUENCE " << globalSettings.getMeta("db.oracle.serialIdSequence.name").getStringValue() << " MINVALUE 1 START WITH 1 INCREMENT BY 1 CACHE 20 NOCYCLE";
	} else {
		str2 << attributes.getMeta("db.oracle.serialIdSequence.sql.create").getStringValue() << endl;
	}
	ret.push_back(str2.str());

	return ret;
}

vector<string> Oracle10gMappingStatementsFeature::dbCreateScriptEnd(const MetadataHolder& attributes)
{
	return vector<string>();
}

vector<string> Oracle10gMappingStatementsFeature::dbDropScriptStart(const MetadataHolder& attributes)
{
	vector<string> ret;
	ostringstream str;
	if (!attributes.hasMeta("db.oracle.oidSequence.sql.drop")) {
		str << "DROP SEQUENCE " << globalSettings.getMeta("db.oracle.oidSequence.name").getStringValue();
	} else {
		str << attributes.getMeta("db.oracle.oidSequence.sql.drop").getStringValue();
	}
	ret.push_back(str.str());
	ostringstream str2;
	if (!attributes.hasMeta("db.oracle.serialIdSequence.sql.drop")) {
		str2 << "DROP SEQUENCE " << globalSettings.getMeta("db.oracle.serialIdSequence.name").getStringValue();
	} else {
		str2 << attributes.getMeta("db.oracle.serialIdSequence.sql.drop").getStringValue();
	}
	ret.push_back(str2.str());

	return ret;
}

vector<string> Oracle10gMappingStatementsFeature::dbDropScriptEnd(const MetadataHolder& attributes)
{
	return vector<string>();
}


vector<string> Oracle10gMappingStatementsFeature::getCreateClassTableScript(const DbType& dbType,  const MetadataHolder& attributes)
{
	if (dbType.getMappingType() == DbType::MAPPING_OBJECT) {
		return objectStatements->getCreateClassTableScript(dbType.getObjectMappingStrategy(), attributes);
	} else {
		return orStatements->getCreateClassTableScript(dbType.getORMappingStrategy(), attributes);
	}
}

void Oracle10gMappingStatementsFeature::getViewBase(const DbType& dbType, const MetadataHolder& attributes, string& ocols, string& otabs)
{
	ostringstream ocolsStr;
	ostringstream otabsStr;
	set<string> processedColumns;
	bool start = true;
	string mainTabName;

	// TODO: kdyz OIDObject bude OO, pak upravit where na typ..... mozna nebude treba - zkusit (ale bylo by vhodno..)

	if (dbType.getMappingType() == DbType::MAPPING_OBJECT) {
		ObjectMappingStrategy& currentStrategy = dbType.getObjectMappingStrategy();
		const Type& t = currentStrategy.getFirstRoot().getType();
		mainTabName = "X";
		otabsStr << t.getMeta("db.table").getStringValue() << " X";
		objectStatements->processAttributesForView(currentStrategy, start, ocolsStr, processedColumns, currentStrategy.getType());
		/*
		for (DbType::AllParentsIterator it2 = dbType.getAllParents().begin(); it2 != dbType.getAllParents().end(); it2++) {
			if (!isOO(**it2)) continue;
			processAttributes(**it2, start, ocolsStr, processedColumns, dbType.getType());
		}*/


	} else {
		ORMappingStrategy& currentStrategy = dbType.getORMappingStrategy();
		if (!currentStrategy.hasDbTable()) {
			// filtered mapping
			mainTabName = currentStrategy.getInsertToType().getType().getMeta("db.table").getStringValue();
		} else {
			mainTabName = currentStrategy.getType().getMeta("db.table").getStringValue();
		}
		otabsStr << mainTabName;
		orStatements->processAttributesForView(currentStrategy, start, ocolsStr, processedColumns, currentStrategy.getType());
	}

	string oidColName = TypeDesc<OidObject>::getType().getAttribute("oid").getMeta("db.column").getStringValue();
	for (DbType::AllParentsIterator it2 = dbType.getAllParents().begin(); it2 != dbType.getAllParents().end(); it2++) {

		// There is no need for this branch because no MAPPING_OBJECT class is present in the parents list
		// (no non-MAPPING_OBJECT class can be inherited from a MAPPING_OBJECT class
		/*if ((*it2)->getMappingType() == DbType::MAPPING_OBJECT) {
			objectStatements->processAttributesForView((*it2)->getObjectMappingStrategy(), start, ocolsStr, processedColumns, dbType.getType());
		} else {*/

			// Do not add inner join on target of filtered mapping (because it is already added as the mainTabName)
			if (dbType.getMappingType() == DbType::MAPPING_OBJECT || !(!dbType.getORMappingStrategy().hasDbTable() && dbType.getORMappingStrategy().getInsertToType() == *(*it2))) {
				string tabName = (*it2)->getType().getMeta("db.table").getStringValue();
				otabsStr << " INNER JOIN " << tabName << " ON (" << mainTabName << "." << oidColName << " = " << tabName << "." << oidColName << ")";
			}

			ORMappingStrategy currentStrategy = (*it2)->getORMappingStrategy();
			orStatements->processAttributesForView(currentStrategy, start, ocolsStr, processedColumns, dbType.getType());
		//}
	}
	ocols = ocolsStr.str();
	otabs = otabsStr.str();
}

vector<string> Oracle10gMappingStatementsFeature::getCreateUpDownViewsScript(const DbType& dbType,  const MetadataHolder& attributes)
{
	ostringstream ostr;
	string ocols;
	string otabs;

	getViewBase(dbType, attributes, ocols, otabs);

	ostr << "SELECT " << ocols << " FROM " << otabs;
	if (DbTypeManager::getDbType(TypeDesc<OidObject>::getType()).getMappingType() == DbType::MAPPING_OBJECT) {
		ostr << " WHERE X.";
	}
	else {
		ostr << " WHERE " << TypeDesc<OidObject>::getType().getMeta("db.table").getStringValue() << ".";
	}
	ostr << TypeDesc<OidObject>::getType().getAttribute("className").getMeta("db.column").getStringValue()
		<< " = '" << dbType.getType().getQualifiedName() << "'";

	MetadataHolder meta;
	meta["db.view"].setStringValue(dbType.getType().getMeta("db.view.sv").getStringValue());
	meta["db.view.select"].setStringValue(ostr.str());

	vector<string> ret;
	ret.push_back(sqlStatements->getCreateViewStatement(meta));

	return ret;
}

vector<string> Oracle10gMappingStatementsFeature::getCreateDownUpViewsScript(const DbType& dbType,  const MetadataHolder& attributes)
{
	ostringstream ostr;

	if (dbType.getType().is<OidObject>()) {
		ostr << "SELECT * FROM " << TypeDesc<OidObject>::getType().getMeta("db.table").getStringValue();
	} else {
		string ocols;
		string otabs;
		ostringstream ounionall;
		const Type& oidType = TypeDesc<OidObject>::getType();
		DbType& oidDbType = DbTypeManager::getDbType(oidType);
		string oidColName = oidType.getAttribute("oid").getMeta("db.column").getStringValue();
		string oidTabName = oidType.getMeta("db.table").getStringValue();
		const Type& type = dbType.getType();

		getViewBase(dbType, attributes, ocols, otabs);
		ostr << "SELECT " << ocols << " FROM " << otabs;

		if (dbType.getMappingType() == DbType::MAPPING_FILTERED) {
			ostr << " WHERE " << oidTabName << "." << oidType.getAttribute("className").getMeta("db.column").getStringValue() <<
				" IN ('" << type.getQualifiedName() << "'";
		} else if (dbType.getMappingType() == DbType::MAPPING_OBJECT) {
			if (globalSettings.hasMeta("db.oracle.workaround.10gOO")) {
				const Type& t = dbType.getObjectMappingStrategy().getFirstRoot().getType();
				ostr << " WHERE X." << oidColName << " IN (SELECT Y." << oidColName << " FROM " << t.getMeta("db.table").getStringValue() <<
					" Y WHERE VALUE(Y) IS OF(" << type.getMeta("db.type").getStringValue() << "))";
			} else {
				ostr << " WHERE VALUE(X) IS OF (" << type.getMeta("db.type").getStringValue() << ")";
			}
		}


		vector<const MetadataHolder*> colMeta;
		for (DbType::PersistentAttributesIterator it = dbType.getPersistentAttributes().begin(); it != dbType.getPersistentAttributes().end(); it++) {
			colMeta.push_back(*it);
		}

		vector<const MetadataHolder*> oidColMeta;
		for (DbType::PersistentAttributesIterator it = oidDbType.getPersistentAttributes().begin(); it != oidDbType.getPersistentAttributes().end(); it++) {
			oidColMeta.push_back(*it);
		}

		for(Type::AllChildrenIterator it = type.getAllChildren().begin(); it != type.getAllChildren().end(); it++) {
			DbType& dbChild = DbTypeManager::getDbType(**it);
			const Type& childType = dbChild.getType();

			if (dbType.getMappingType() == DbType::MAPPING_FILTERED) {
				ostr << ", '" << childType.getQualifiedName() << "'";
			}

			if (dbChild.getMappingType() != DbType::MAPPING_HORIZONTAL) continue;

			string childTableName = dbChild.getType().getMeta("db.table").getStringValue();

			string columns = sqlStatements->getColumns(childType, colMeta, true, false, false, false, Oracle10gSqlStatementsFeature::TABLEASVIEW_COLS, childTableName + ".");
			string oidColumns = sqlStatements->getColumns(oidType, oidColMeta, true, false, false, false, Oracle10gSqlStatementsFeature::TABLEASVIEW_COLS, oidTabName + ".");

			ounionall << " UNION ALL ";
			ounionall << "SELECT " << columns << ", " << oidColumns << " FROM " << childTableName <<
				" INNER JOIN " << oidTabName << " ON (" << childTableName << "." << oidColName <<
				" = " << oidTabName << "." << oidColName << ")";
		}

		if (dbType.getMappingType() == DbType::MAPPING_FILTERED) {
			ostr << ")";
		}

		ostr << ounionall.str();
	}

	MetadataHolder meta;
	meta["db.view"].setStringValue(dbType.getType().getMeta("db.view.pv").getStringValue());
	meta["db.view.select"].setStringValue(ostr.str());

	vector<string> ret;
	ret.push_back(sqlStatements->getCreateViewStatement(meta));

	return ret;

/*
	if (dbType.getType().is<OidObject>()) {
		ostr << "SELECT * FROM " << TypeDesc<OidObject>::getType().getMeta("db.table").getStringValue();
	} else {

		vector<const MetadataHolder*> colMeta;
		for (DbType::PersistentAttributesIterator it = dbType.getAllPersistentAttributes().begin(); it != dbType.getAllPersistentAttributes().end(); it++) {
			colMeta.push_back(*it);
		}
		string columns = sqlStatements->getColumns(dbType.getType(), colMeta, true, false, false, false, true);


		// TODO: dodelat getAllChildren !!!!!!!!
		ostr << "SELECT " << columns << " FROM " << dbType.getType().getMeta("db.view.sv").getStringValue();
		const Type::AllChildrenList& children = dbType.getType().getAllChildren();
		for (Type::AllChildrenIterator it = children.begin(); it != children.end(); it++) {
			ostr << " UNION ALL ";
			ostr << "SELECT " << columns << " FROM " << (**it).getMeta("db.view.sv").getStringValue();
		}
	}

	MetadataHolder meta;
	meta["db.view.name"].setStringValue(dbType.getType().getMeta("db.view.pv").getStringValue());
	meta["db.view.select"].setStringValue(ostr.str());

	vector<string> ret;
	ret.push_back(sqlStatements->getCreateViewStatement(meta));
*/
}

vector<string> Oracle10gMappingStatementsFeature::getDropClassTableScript(const DbType& dbType,  const MetadataHolder& attributes)
{
	if (dbType.getMappingType() == DbType::MAPPING_OBJECT) {
		return objectStatements->getDropClassTableScript(dbType.getObjectMappingStrategy(), attributes);
	} else {
		return orStatements->getDropClassTableScript(dbType.getORMappingStrategy(), attributes);
	}
}

vector<string> Oracle10gMappingStatementsFeature::getDropUpDownViewsScript(const DbType& dbType,  const MetadataHolder& attributes)
{
	MetadataHolder meta;
	meta["db.view"].setStringValue(dbType.getType().getMeta("db.view.sv").getStringValue());
	vector<string> ret;
	ret.push_back(sqlStatements->getDropViewStatement(meta));
	return ret;
}

vector<string> Oracle10gMappingStatementsFeature::getDropDownUpViewsScript(const DbType& dbType,  const MetadataHolder& attributes)
{
	MetadataHolder meta;
	meta["db.view"].setStringValue(dbType.getType().getMeta("db.view.pv").getStringValue());
	vector<string> ret;
	ret.push_back(sqlStatements->getDropViewStatement(meta));
	return ret;
}



/*string Oracle10gMappingStatementsFeature::getWhere(const DbType::KeyAttributesSet& keys, string metaColName)
{
	ostringstream out;
	int i = 1;
	for (DbType::KeyAttributesIterator it = keys.begin(); it != keys.end(); it++)
	{
		if (it != keys.begin()) {
			out << " AND ";
		}

		out << (*it)->getMeta(metaColName).getStringValue() << " = :key" << i;
		i++;
	}
	return out.str();
}*/


// TODO: use sqlStatementsFeature and add order by to it
string Oracle10gMappingStatementsFeature::getQueryStatement(const DbType& dbType, const MetadataHolder& attributes)
{
	ostringstream out;
	out << "SELECT ";
	bool viewCols;
	if (!dbType.isOidManaged()) {
		viewCols = false;
	} else if (!attributes.hasFeature("db.query.from")) {
		viewCols = true;
	} else {
		viewCols = attributes.hasFeature("db.query.useViewCols");
	}

	if (dbType.getType().hasFeature("db.transient")) {
		for (DbType::PersistentAttributesIterator it = dbType.getAllPersistentAttributes().begin(); it != dbType.getAllPersistentAttributes().end(); it++) {
			if (it != dbType.getAllPersistentAttributes().begin()) {
				out << ", ";
			}
			const Attribute& attr = **it;
			if (attr.hasMeta("db.column.sql")) {
				out << attr.getMeta("db.column.sql").getStringValue() << " AS ";
			}
			if (attributes.hasMeta("db.query.colPrefix")) {
				out << attributes.getMeta("db.query.colPrefix").getStringValue() << ".";
			}
			if (viewCols) {
				out << attr.getMeta("db.viewColumn").getStringValue();
			} else {
				out << attr.getMeta("db.column").getStringValue();
			}
		}
	} else {
		for (DbType::KeyAttributesIterator it = dbType.getKeyAttributes().begin(); it != dbType.getKeyAttributes().end(); it++) {
			if (it != dbType.getKeyAttributes().begin()) {
				out << ", ";
			}
			if (attributes.hasMeta("db.query.colPrefix")) {
				out << attributes.getMeta("db.query.colPrefix").getStringValue() << ".";
			}
			const Attribute& attr = **it;
			if (viewCols) {
				out << attr.getMeta("db.viewColumn").getStringValue();
			} else {
				out << attr.getMeta("db.column").getStringValue();
			}
		}
	}

	out << " FROM ";

	if (attributes.hasMeta("db.query.from")) {
		out << attributes.getMeta("db.query.from").getStringValue();
	} else {
		if (viewCols) {
			out << dbType.getType().getMeta("db.view.pv").getStringValue();
		} else {
			out << dbType.getType().getMeta("db.table").getStringValue();
		}
	}

	if (!attributes.hasMeta("db.query.predef.all")) {
		if (attributes.hasMeta("db.query.predef.none")) {
			out << " WHERE 0<>0";
		} else if (attributes.hasMeta("db.query.where")) {
			out << " WHERE " << attributes.getMeta("db.query.where").getStringValue();
		}
	}

	if (attributes.hasMeta("db.query.orderBy")) {
		out << " ORDER BY " << attributes.getMeta("db.query.orderBy").getStringValue();
	}
	return out.str();

}
