#ifndef ORACLE10GMAPPINGSTATEMENTSFEATURE_H_
#define ORACLE10GMAPPINGSTATEMENTSFEATURE_H_

#include "iopclib/db/mappingStatementsFeature.h"
#include "iopcdriver_oracle10g/oracle10gSqlStatementsFeature.h"
#include "iopcdriver_oracle10g/oracle10gTypeMappingFeature.h"
#include "iopclib/types/dbType.h"
#include "iopclib/types/orMappingStrategy.h"
#include "iopclib/db/statementCache.h"
#include "oracle10gObjectStatementsFeature.h"
#include "oracle10gORStatementsFeature.h"
#include <sstream>
#include <set>
#include <string>

namespace iopc {

class Type;

class Oracle10gMappingStatementsFeature : public MappingStatementsFeature
{
private:
	Oracle10gSqlStatementsFeature* sqlStatements;
	Oracle10gTypeMappingFeature* typeMapping;
	Oracle10gORStatementsFeature* orStatements;
	Oracle10gObjectStatementsFeature* objectStatements;

	//std::string getWhere(const DbType::KeyAttributesSet& keys, std::string metaColName);
	//void processAttributes(ORMappingStrategy& currentStrategy, bool& start, std::ostringstream& ocolstr, std::set<std::string>& processedColumns, const Type& mainType);
	void getViewBase(const DbType& dbType, const MetadataHolder& attributes, std::string& ocols, std::string& otabs);

	StatementCache& cache;
public:
	Oracle10gMappingStatementsFeature(StatementCache& cache) : cache(cache) {};
	inline void init(SqlStatementsFeature& sqlStatements,
		TypeMappingFeature& typeMapping,
		Oracle10gORStatementsFeature& orStatements,
		Oracle10gObjectStatementsFeature& objectStatements) {
		this->sqlStatements = dynamic_cast<Oracle10gSqlStatementsFeature*>(&sqlStatements);
		this->typeMapping = dynamic_cast<Oracle10gTypeMappingFeature*>(&typeMapping);
		this->orStatements = dynamic_cast<Oracle10gORStatementsFeature*>(&orStatements);
		this->objectStatements = dynamic_cast<Oracle10gObjectStatementsFeature*>(&objectStatements);
	}

	virtual std::string getSelectTypeStatement(const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual std::string getSelectSerialIdStatement(const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual std::string getNextOidStatement(const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual std::string getNextSerialIdStatement(const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true);
	virtual std::vector<std::string> dbCreateScriptStart(const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	virtual std::vector<std::string> dbCreateScriptEnd(const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	virtual std::vector<std::string> dbDropScriptStart(const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	virtual std::vector<std::string> dbDropScriptEnd(const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	virtual std::vector<std::string> getCreateClassTableScript(const DbType& dbType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	virtual std::vector<std::string> getDropClassTableScript(const DbType& dbType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	virtual std::vector<std::string> getCreateUpDownViewsScript(const DbType& dbType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	virtual std::vector<std::string> getCreateDownUpViewsScript(const DbType& dbType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	virtual std::vector<std::string> getDropUpDownViewsScript(const DbType& dbType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	virtual std::vector<std::string> getDropDownUpViewsScript(const DbType& dbType, const MetadataHolder& attributes = MetadataHolder::emptyMeta);
	virtual std::string getQueryStatement(const DbType& dbType, const MetadataHolder& attributes);

};


}


#endif /*ORACLE10GMAPPINGSTATEMENTSFEATURE_H_*/
