#ifndef CACHEDDATABASE_H_
#define CACHEDDATABASE_H_

#include "iopcdb/database.h"
#include "../cache/cache.h"

namespace iopc {


/**
 * A Database class decorator that creates cached connections.
 * 
 * First pass a prototype of desired cache to the constructor.
 * Consequent calls to getConnection will clone the prototype,
 * create a new CachedConnection instance and pass this clone to it.
 */
class DLL_EXTERNAL CachedDatabase : public Database {

private:
	Database* inner;
	Cache* cache;

public:

	CachedDatabase(Database* database, Cache* cachePrototype);

	/**
	 * Deletes the cache prototype and the inner instance of Database
	 */
	virtual ~CachedDatabase();
	
	virtual void commit();

	virtual void rollback();

	virtual Connection* getConnection(const std::string& connStr);

	virtual void returnConnection(Connection* connection);

	virtual void disconnect();
	
	virtual DatabaseImpl* getImplementation();
	
	virtual Database* getBasicDatabase();
	
	virtual Driver& getDriver();

}; // class


} // namespace


#endif /*CACHEDDATABASE_H_*/
