#ifndef DBPTR_H_
#define DBPTR_H_

#include "persIdentification.h"
#include "iopccommon/exceptionBase.h"
#include "cachePtr.h"
#include "constCachePtr.h"
#include "dbPtrBase.h"
#include "iopcmeta/typeDesc.h"
#include "databaseObject.h"
//#include "oidObject.h"

namespace iopc {

template<typename T> class CachePtr;
//template<typename T> class ConstCachePtr;

/**
 * @brief Template of classes representing typed database pointers.
 *
 */
template<typename T> class DbPtr: public DbPtrBase {
/*friend class CachePtr<T>;
friend class ConstCachePtr<T>;*/
public:
	DbPtr(/*bool createInstance = true*/)
		throw ();

	DbPtr(const DbPtr<T> &src)
		throw ();

	/*DbPtr(Connection &conn, const QueRefProto &query)
		throw ();*/
	DbPtr(Cursor& cur)
		throw ();

	DbPtr(CachedConnection* conn, const OidType& oid, bool loadRealType = true)
		throw ();

	DbPtr(CachedConnection* conn, const KeyValues& keyVals)
		throw ();

	DbPtr(CachedConnection* conn)
		throw ();

	~DbPtr()
		throw ();

	DbPtr<T> &operator=(const DbPtr<T> &src)
		throw ();


	/** Creates new reference to transient clone (also new) of the referenced instance. */
/*	DbPtr<T>* transClone() const
		throw (PoliteExceptionLock, PoliteExceptionNoMemory);*/

	/** Makes the referenced object persistent. */
	CachePtr<T> bePersistent(CachedConnection* conn);


	/**
	 * @brief Dereferences database pointer.
	 *
	 * @return Instance representing locked copy of referenced object.
	 */
	inline CachePtr<T> operator*() const;

	/**
	 * @brief Dereferences database pointer.
	 *
	 * @return Instance representing locked copy of referenced object.
	 */
	CachePtr<T> operator->() const;

	/**
	 * @brief Dereferences database pointer.
	 *
	 * @return Instance representing locked copy of referenced object.
	 */
	CachePtr<T> getCachePtr() const;

	/**
	 * @brief Dereferences database pointer for reading.
	 *
	 * @return Instance representing copy locked for reading (shared lock).
	 */
	ConstCachePtr<T> getConstCachePtr() const;

private:
/*	CachePtr<T> getNewCachePtr();
	ConstCachePtr<T> getNewConstCachePtr();*/

	CountedPtr<T> objectPtr;



protected:
	CountedPtr<T> getObjectPtr() const { return objectPtr; }
	/** Ctor used for cloning */
/*	inline DbPtr(T& objectPtr)
		throw ();*/

	virtual bool isObjectPtrNull() const { return objectPtr.isNull(); }
	virtual bool isObjectPtrUnique() const { return objectPtr.unique(); }
	virtual DatabaseObject& getObject() const { return dynamic_cast<DatabaseObject&>(*objectPtr); }
	virtual void setObjectPtrNull(bool deleteObject = true) { objectPtr.allowDelete(deleteObject); objectPtr = CountedPtr<T>(); }
};

// TODO: Asi bude na typu nejaky operator *, ktery vrati jeho reprezentanta.. nebo tak neco...
//  Vsude jsem ubral z *T::getType hvezdicku


template<typename T> DbPtr<T>::DbPtr(/*bool createInstance*/)
	throw ()
: DbPtrBase(TypeDesc<T>::getType()/*, createInstance*/), objectPtr(CountedPtr<T>(dynamic_cast<T*>(TypeDesc<T>::getType().newObjectInstance())))
{
	callOnIopcCreate(objectPtr.get());
	//objectPtr->onIopcCreate();
	//setNewInstance(dynamic_cast<DatabaseObject*>(TypeDesc<T>::getType().newObjectInstance()));
}

template<typename T> DbPtr<T>::DbPtr(Cursor& cur)
	throw ()
: DbPtrBase(TypeDesc<T>::getType()), objectPtr()
{
	DatabaseObject* obj = getPersId().load(cur);
	if (obj == NULL) {
		return; // isValid() will return false
	} else {
		objectPtr = CountedPtr<T>(dynamic_cast<T*>(obj));
	}
}

template<typename T> DbPtr<T>::DbPtr(const DbPtr<T>& src)
	throw ()
: DbPtrBase(src), objectPtr(src.objectPtr)
{
	//src.objectPtr;
	/*if (src.getState() == PERSISTENT) {
		objectPtr = CountedPtr<T>();
	} else {
		objectPtr = src.objectPtr;
	}*/
}

template<typename T> DbPtr<T>::DbPtr(CachedConnection* conn, const OidType& oid, bool loadRealType)
	throw ()
: DbPtrBase(TypeDesc<T>::getType(), conn, oid, loadRealType), objectPtr()
{
}

template<typename T> DbPtr<T>::DbPtr(CachedConnection* conn, const KeyValues& keyVals)
	throw ()
: DbPtrBase(TypeDesc<T>::getType(), conn, keyVals), objectPtr()
{
}

/*template<typename T> inline DbPtr<T>::DbPtr(T &objectPtr)
	throw ()
: DbPtrBase(*T::getType(), false)
{
	ASSERT(persId.keyValues == NULL && conn == NULL);
	this->objectPtr = &objectPtr;
}*/

/*template<typename T> DbPtr<T>::DbPtr(Connection &conn, const QueRefProto &query)
	throw (PoliteExceptionNoMemory)
: DbPtrBase(*T::StPrototype(), true)
{
	 *this = ++Result<T>(conn, query);
}*/

template<typename T> DbPtr<T>::~DbPtr()
	throw ()
{
	//deleteObject();
}

template<typename T> DbPtr<T> &DbPtr<T>::operator=(const DbPtr<T>& src)
	throw ()
{
	if (this == &src)
		return *this;

	objectPtr = src.objectPtr;
	/*if (src.getState() == PERSISTENT) {
		objectPtr = CountedPtr<T>();
	} else {
		objectPtr = src.objectPtr;
	}*/

	replaceWith(src);

	return *this;
}

/*template<typename T> inline DbPtr<T> *DbPtr<T>::transientClone() const
	throw ()
{
	Object *obj = this->getConstCachePtr().getObjectPtr()->clone();
	obj->setTransient();
	obj->setDirty(false);
	return new DbPtr<T>(*(T *)obj);
}*/

template<typename T> CachePtr<T> DbPtr<T>::bePersistent(CachedConnection* conn)
{
	CacheLock& cLock = DbPtrBase::bePersistent(conn);
	propagateConnectionToAttributes(cLock.getObject());
	return CachePtr<T>(cLock);
}

template<typename T> CachePtr<T> DbPtr<T>::operator*() const
{
	return getCachePtr();
}

template<typename T> CachePtr<T> DbPtr<T>::operator->() const
{
	return getCachePtr();
}

template<typename T> CachePtr<T> DbPtr<T>::getCachePtr() const
{
	switch (getState()) {
		case PERSISTENT:
		{
				CachePtr<T> ret = CachePtr<T>(getConnection(), getPersId());
				DbPtrBase::propagateConnectionToAttributes(*ret.getObjectPtr().get());
				return ret;
			break;
		}
		case TRANSIENT:
		{
				if (getObjectPtr().isNull()) {
					IOPC_LOG_THROW(IopcInvalidStateException("Object was deleted or does not exist"));
				}
				return CachePtr<T>(getObjectPtr());
			break;
		}
		default:
			IOPC_UNREACHABLE;
	}
	IOPC_UNREACHABLE;
}

template<typename T> ConstCachePtr<T> DbPtr<T>::getConstCachePtr() const
{
	switch (getState()) {
		case PERSISTENT:
		{
				ConstCachePtr<T> ret = ConstCachePtr<T>(getConnection(), getPersId());
				DbPtrBase::propagateConnectionToAttributes(*ret.getObjectPtr().get());
				return ret;
			break;
		}
		case TRANSIENT:
		{
				if (getObjectPtr().isNull()) {
					IOPC_LOG_THROW(IopcInvalidStateException("Object was deleted or does not exist"));
				}
				return ConstCachePtr<T>(getObjectPtr());
			break;
		}
		default:
			IOPC_UNREACHABLE;
	}
	IOPC_UNREACHABLE;
}

}

#endif /*DBPTR_H_*/
