#include "dbTypeManager.h"
#include "iopccommon/globalSettings.h"
#include "iopcdb/cursor.h"
#include "iopcdb/driverManager.h"
#include "iopccommon/util.h"
#include "../db/mappingStatementsFeature.h"
#include <string>

using namespace std;

namespace iopc {

DbTypeManager& DbTypeManager::getInstance()
{
	static DbTypeManager instance;
	return instance;
}

void DbTypeManager::initInst()
{
	//initSql();
	
	TypeManager::TypesMap types = TypeManager::getTypesMap();
	for (TypeManager::TypesIterator it = types.begin(); it != types.end(); it++) {
		if (it->second->isTypeOf<DatabaseObject>() && it->second->isNot<DatabaseObject>()) {
			dbTypes[it->second]; // just create instances;
		}
	}
	for (map<const Type*, DbType>::iterator it = dbTypes.begin(); it != dbTypes.end(); it++) {
		it->second.init(*it->first);
	}
	for (map<const Type*, DbType>::iterator it = dbTypes.begin(); it != dbTypes.end(); it++) {
		it->second.postInit();
	}
}

DbTypeManager::DbTypeManager()
{
}

DbTypeManager::~DbTypeManager()
{
	/*for(map<const Type*, DbType*>::iterator it = dbTypes.begin(); it != dbTypes.end(); it++) {
		delete it->second;sqlStatements
	}*/
}

DbType& DbTypeManager::getDbTypeInst(const Type& t)
{
	map<const Type*, DbType>::iterator it = dbTypes.find(&t);
	if(it == dbTypes.end()) {
		IOPC_LOG_THROW(IopcNotFoundException(string("No DbType for the type ") + t.getQualifiedName() + " found."));
		/*DbType* dbType = new DbType(t);
		dbTypes.insert(std::make_pair<const Type*, DbType*>(&t, dbType));
		return *dbType;*/
		//return dbTypes[&t];
	} else {
		//DbType& retType = *(*it).second;
		return it->second;
	}
}


/*void DbTypeManager::initSql()
{
	vector<string> driversStrList;
	string drivers = globalSettings.getMeta("db.initSqlForDrivers").getStringValue();
	StringUtil::splitString(drivers, ";", driversStrList);
	
	for (vector<string>::const_iterator it = driversStrList.begin(); it != driversStrList.end(); it++) {
		Driver* driver = &DriverManager::getDriver(*it);
		if (!driver->supportsFeature("objectMapping")) {
			continue;
		}
		driversList.push_back(driver);
		
		sqlStatements[driver].selectTypeStatement = driver->getFeature<ObjectMappingFeature>().getSelectTypeStatement();
		sqlStatements[driver].getNextOidStatement = driver->getFeature<ObjectMappingFeature>().getNextOidStatement();
		sqlStatements[driver].getNextSerialIdStatement = driver->getFeature<ObjectMappingFeature>().getNextSerialIdStatement();
		sqlStatements[driver].selectSerialIdStatement = driver->getFeature<ObjectMappingFeature>().getSelectSerialIdStatement();
	}
}*/

OidType DbTypeManager::getNextOidInst(Connection& conn)
{
	OidType oid;
	string sql = conn.getDriver().getFeature<MappingStatementsFeature>().getNextOidStatement();
	//LogWriter::logSql(sql);
	Cursor* cur = conn.sql(sql);
	cur->addOutParam(1, &oid, TypeDesc<OidType>::getType());
	cur->execute();
	//cur->preFetchBind(1, &oid, sizeof(OidType), TypeDesc<OidType>::getType());
	cur->fetchNext();
	cur->close();
	conn.returnCursor(cur);
	return oid;
}

SerialIdType DbTypeManager::getNextSerialIdInst(Connection& conn)
{
	SerialIdType serialId;
	string sql = conn.getDriver().getFeature<MappingStatementsFeature>().getNextSerialIdStatement();
	//LogWriter::logSql(sql);
	Cursor* cur = conn.sql(sql);
	cur->addOutParam(1, &serialId, TypeDesc<SerialIdType>::getType());
	cur->execute();
	//cur->preFetchBind(1, &oid, sizeof(OidType), TypeDesc<OidType>::getType());
	cur->fetchNext();
	cur->close();
	conn.returnCursor(cur);
	return serialId;
}
	
const Type& DbTypeManager::getTypeInst(Connection& conn, const OidType oid)
{
	IOPC_FUNC_INVOKED;
	
	EString className;
	string sql = conn.getDriver().getFeature<MappingStatementsFeature>().getSelectTypeStatement();
	//LogWriter::logSql(selectTypeStatement);
	
	Cursor* cur = conn.sql(sql);
	cur->addInParam(1, &oid, TypeDesc<OidType>::getType());
	cur->addEnhancedOutParam(1, className, TypeDesc<OidObject>::getType().getAttribute(string("className")));
	cur->execute();
	//cur->preFetchBind(1, className, len);
	if (!cur->fetchNext())
	{
		IOPC_LOG_THROW(IopcNotFoundException(string("No such object found. Oid=") + oid.toString()));
	}
	//className.updateFromExternalized();
	cur->close();
	conn.returnCursor(cur);
	
	return TypeManager::getType(className);
}

SerialIdType DbTypeManager::getSerialIdInst(Connection& conn, const OidType oid)
{
	IOPC_FUNC_INVOKED;
	
	SerialIdType serialId;
	string sql = conn.getDriver().getFeature<MappingStatementsFeature>().getSelectSerialIdStatement();
	
	Cursor* cur = conn.sql(sql);
	cur->addInParam(1, &oid, TypeDesc<OidType>::getType());
	cur->addOutParam(1, &serialId, TypeDesc<SerialIdType>::getType());
	cur->execute();
	//cur->preFetchBind(1, className, len);
	if (!cur->fetchNext())
	{
		IOPC_LOG_THROW(IopcNotFoundException(string("No such object found. Oid=") + oid.toString()));
	}
	//className.updateFromExternalized();
	cur->close();
	conn.returnCursor(cur);
	
	return serialId;
}

	
}
