#include "scriptsGenerator.h"
#include "iopcmeta/typeManager.h"
#include "iopcmeta/type.h"
#include "dbType.h"
#include "dbTypeManager.h"
#include <vector>
#include "iopcdb/driver.h"
#include "../db/mappingStatementsFeature.h"

using namespace std;

namespace iopc {

vector<string> ScriptsGenerator::getDbDropScript(const Driver& driver)
{
	vector<string> out;
	vector<string> script;
	vector<const Type*> sortedTypes;
	MappingStatementsFeature& omf = driver.getFeature<MappingStatementsFeature>();
	
	TypeManager::topologicalSort(TypeDesc<OidObject>::getType(), sortedTypes);
	
	script = omf.dbDropScriptStart();
	out.insert(out.end(), script.begin(), script.end());
	
	// First views, then tables
	
	// PV
	for (vector<const Type*>::const_iterator it = sortedTypes.begin(); it != sortedTypes.end(); it++) {
		if ((*it)->hasFeature("scripts.exclude") || (*it)->hasFeature("db.transient")) continue;
		const DbType& dbType = DbTypeManager::getDbType(**it);

		script = omf.getDropDownUpViewsScript(dbType);
		out.insert(out.end(), script.begin(), script.end());
	}
	// SV
	for (vector<const Type*>::reverse_iterator it = sortedTypes.rbegin(); !(it == sortedTypes.rend()); it++) {
		if ((*it)->hasFeature("scripts.exclude") || (*it)->hasFeature("db.transient")) continue;
		const DbType& dbType = DbTypeManager::getDbType(**it);

		script = omf.getDropUpDownViewsScript(dbType);
		out.insert(out.end(), script.begin(), script.end());
	}
	
	for (vector<const Type*>::reverse_iterator it = sortedTypes.rbegin(); it != sortedTypes.rend(); it++) {
		if ((*it)->hasFeature("scripts.exclude") || (*it)->hasFeature("db.transient")) continue;
		const DbType& dbType = DbTypeManager::getDbType(**it);
		
		script = omf.getDropClassTableScript(dbType);
		if (script.size() != 0) {
			out.insert(out.end(), script.begin(), script.end());
		}
	}
	// Non-oid objects
	for (DbTypeManager::DbTypesIterator it = DbTypeManager::getDbTypesMap().begin(); it != DbTypeManager::getDbTypesMap().end(); it++) {
		if (it->second.getType().hasFeature("scripts.exclude") || it->second.getType().hasFeature("db.transient")) continue;
		if (it->second.getType().isTypeOf<OidObject>()) continue;
		script = omf.getDropClassTableScript(it->second);
		if (script.size() != 0) {
			out.insert(out.end(), script.begin(), script.end());
		}
	}
	
	script = omf.dbDropScriptEnd();
	out.insert(out.end(), script.begin(), script.end());
	
	return out;
	
}
	
vector<string> ScriptsGenerator::getDbCreateScript(const Driver& driver)
{
	vector<string> out;
	vector<string> script;
	MappingStatementsFeature& omf = driver.getFeature<MappingStatementsFeature>();
	vector<const Type*> sortedTypes;
	
	TypeManager::topologicalSort(TypeDesc<OidObject>::getType(), sortedTypes);
	
	script = omf.dbCreateScriptStart();
	out.insert(out.end(), script.begin(), script.end());
	
	// First tables, then views
	for (vector<const Type*>::const_iterator it = sortedTypes.begin(); it != sortedTypes.end(); it++) {
		if ((*it)->hasFeature("scripts.exclude") || (*it)->hasFeature("db.transient")) continue;
		const DbType& dbType = DbTypeManager::getDbType(**it);
		
		script = omf.getCreateClassTableScript(dbType);
		if (script.size() != 0) {
			out.insert(out.end(), script.begin(), script.end());
		}
	}
	
	// SV
	for (vector<const Type*>::const_iterator it = sortedTypes.begin(); it != sortedTypes.end(); it++) {
		if ((*it)->hasFeature("scripts.exclude") || (*it)->hasFeature("db.transient")) continue;
		const DbType& dbType = DbTypeManager::getDbType(**it);

		script = omf.getCreateUpDownViewsScript(dbType);
		out.insert(out.end(), script.begin(), script.end());
	}
	// PV
	for (vector<const Type*>::reverse_iterator it = sortedTypes.rbegin(); !(it == sortedTypes.rend()); it++) {
		if ((*it)->hasFeature("scripts.exclude") || (*it)->hasFeature("db.transient")) continue;
		const DbType& dbType = DbTypeManager::getDbType(**it);

		script = omf.getCreateDownUpViewsScript(dbType);
		out.insert(out.end(), script.begin(), script.end());
	}
	
	// Non-oid objects
	for (DbTypeManager::DbTypesIterator it = DbTypeManager::getDbTypesMap().begin(); it != DbTypeManager::getDbTypesMap().end(); it++) {
		if (it->second.getType().hasFeature("scripts.exclude") || it->second.getType().hasFeature("db.transient")) continue;
		if (it->second.getType().isTypeOf<OidObject>()) continue;
		
		script = omf.getCreateClassTableScript(it->second);
		if (script.size() != 0) {
			out.insert(out.end(), script.begin(), script.end());
		}
	}
	
	
	script = omf.dbCreateScriptEnd();
	out.insert(out.end(), script.begin(), script.end());
	
	return out;
}


	
}
