#ifndef ENHANCEDNUMBER_H_
#define ENHANCEDNUMBER_H_

#include "enhancedTypeBase.h"
#include "iopccommon/defs.h"
#include "iopccommon/trace.h"
#include "typeDesc.h"
#include <sstream>
#include <iostream>
#include <string>


namespace iopc {

// TODO: implement rest of the operators

template<typename T> class DLL_EXTERNAL ENumber : public EnhancedTypeBase {
private:
	T value;
	T copy;

	bool updateValue(T newVal);
public:
	ENumber();
	ENumber(const T v);
	ENumber(const ENumber<T>& src);

	// Operators

	operator T() const;
	//operator T&();  // Do not allow obtaining reference to wrapped value

	ENumber<T>& operator= (const ENumber<T>& number);
	ENumber<T>& operator= (const T val);

	ENumber<T>& operator++();
	ENumber<T>& operator++(int);

	ENumber<T>& operator--();
	ENumber<T>& operator--(int);

	ENumber<T>& operator+=(const T val);
	ENumber<T>& operator-=(const T val);
	ENumber<T>& operator*=(const T val);
	ENumber<T>& operator/=(const T val);

  	virtual const Type& getType() const;
  	virtual const TypeDesc<ENumber<T> >& getTypeDesc() const;

  	/**
	 * Creates a copy of this object's value and returns a pointer to it.
	 */
	virtual void* externalizeCopy(unsigned int maxlength = 0);

	/**
	 * Creates an externalized empty buffer for obtaining the value.
	 */
	virtual void* externalizeEmptyBuffer(unsigned int length = 0);

	/**
	 * Updates this object's value from the externalized buffer
	 */
	virtual void updateFromExternalized();

	virtual unsigned int getBufferSize();

	virtual const Type& getInnerType() const;

	virtual std::string toString() const;

 	virtual Object* clone() const;

 	virtual int compare(const Object& o) const;
};

template<typename T> int ENumber<T>::compare(const Object& o) const
{
	const Type& type1 = TypeManager::getTypeByTypeInfo(&typeid(*this));
	const Type& type2 = TypeManager::getTypeByTypeInfo(&typeid(o));

	if (type1 != type2) {
		IOPC_LOG_THROW(IopcIncorrectUsageException(std::string("Can't compare ENumber with type ") + type2.getQualifiedName()));
	}
	T cmpVal = (dynamic_cast<const ENumber<T>& >(o)).value;

	if (value < cmpVal) {
		return -1;
	} else if (value > cmpVal) {
		return 1;
	} else {
		return 0;
	}
}

template<typename T> Object* ENumber<T>::clone() const
{
	return new ENumber<T>(*this);
}

template<typename T> std::string ENumber<T>::toString() const
{
	if (isNull()) {
		return "null";
	}
	else {
		std::ostringstream str;
		str << value;
		return str.str();
	}
}

template<typename T> const Type& ENumber<T>::getType() const
{
	return TypeDesc<ENumber<T> >::getType();
}

template<typename T> const TypeDesc<ENumber<T> >& ENumber<T>::getTypeDesc() const
{
	return TypeDesc<ENumber<T> >::getType();
}


template<typename T> ENumber<T>::ENumber()
{
	value = 0;
}

template<typename T> ENumber<T>::ENumber(const T val)
{
	resetNull();
	value = val;
}

template<typename T> ENumber<T>::ENumber(const ENumber<T>& number)
{
	setNull(number.isNull());
	value = number.value;
}

template<typename T> bool ENumber<T>::updateValue(T newVal)
{

	if (value == newVal) {
		return false;
	}
	resetNull();
	value = newVal;
	setDirty();
	return true;
}

template<typename T> ENumber<T>::operator T() const
{

	return value;
}

/*template<typename T> ENumber<T>::operator T&()
{

	return value;
}*/

template<typename T> ENumber<T>& ENumber<T>::operator=(const ENumber<T>& number)
{
	if (isNull() && number.isNull()) return *this;
	setNull(number.isNull());
	if (isNull()) {
		setDirty();
		return *this;
	}
	updateValue(number.value);

	return *this;
}

template<typename T> ENumber<T>& ENumber<T>::operator=(const T val)
{

	updateValue(val);

	return *this;
}

template<typename T> ENumber<T>& ENumber<T>::operator++()
{

	updateValue(value+1);

	return *this;
}

template<typename T> ENumber<T>& ENumber<T>::operator++(int)
{

	updateValue(value+1);

	return *this;
}

template<typename T> ENumber<T>& ENumber<T>::operator--()
{

	updateValue(value-1);

	return *this;
}

template<typename T> ENumber<T>& ENumber<T>::operator--(int)
{

	updateValue(value-1);

	return *this;
}

template<typename T> ENumber<T>& ENumber<T>::operator+=(const T val)
{

	updateValue(value + val);

	return *this;
}

template<typename T> ENumber<T>& ENumber<T>::operator-=(const T val)
{

	updateValue(value - val);

	return *this;
}

template<typename T> ENumber<T>& ENumber<T>::operator*=(const T val)
{

	updateValue(value * val);

	return *this;
}


template<typename T> ENumber<T>& ENumber<T>::operator/=(const T val)
{

	updateValue(value / val);

	return *this;
}

template<typename T> void* ENumber<T>::externalizeCopy(unsigned int maxlength)
{
	copy = value;
	return &copy;
}

template<typename T> void ENumber<T>::updateFromExternalized()
{
	updateValue(copy);
}

template<typename T> const Type& ENumber<T>::getInnerType() const
{
	return TypeDesc<T>::getType();
}

template<typename T> void* ENumber<T>::externalizeEmptyBuffer(unsigned int length)
{
	// do nothing. the buffer is allocated statically
	return &copy;
}

template<typename T> unsigned int ENumber<T>::getBufferSize()
{
	return sizeof(T);
}


typedef DLL_EXTERNAL ENumber<int> EInt;
typedef DLL_EXTERNAL ENumber<unsigned int> EUInt;
typedef DLL_EXTERNAL ENumber<char> EChar;
typedef DLL_EXTERNAL ENumber<unsigned char> EUChar;
typedef DLL_EXTERNAL ENumber<signed char> ESChar;
typedef DLL_EXTERNAL ENumber<short> EShort;
typedef DLL_EXTERNAL ENumber<unsigned short> EUShort;
typedef DLL_EXTERNAL ENumber<long> ELong;
typedef DLL_EXTERNAL ENumber<unsigned long> EULong;
typedef DLL_EXTERNAL ENumber<float> EFloat;
typedef DLL_EXTERNAL ENumber<double> EDouble;
typedef DLL_EXTERNAL ENumber<long double> ELDouble;
typedef DLL_EXTERNAL ENumber<wchar_t> EWCharT;
typedef DLL_EXTERNAL ENumber<bool> EBool;


///////////// Type descriptors ///////////////


SIMPLE_TYPE_DEFINITION("EInt", EInt, "iopc", "iopc::EInt", IOPC_TYPE_EINT, IOPC_TYPECLASS_ENHANCED);
SIMPLE_TYPE_DEFINITION("EUInt", EUInt, "iopc", "iopc::EUInt", IOPC_TYPE_EUINT, IOPC_TYPECLASS_ENHANCED);
SIMPLE_TYPE_DEFINITION("EChar", EChar, "iopc", "iopc::EChar", IOPC_TYPE_ECHAR, IOPC_TYPECLASS_ENHANCED);
SIMPLE_TYPE_DEFINITION("EUChar", EUChar, "iopc", "iopc::EUChar", IOPC_TYPE_EUCHAR, IOPC_TYPECLASS_ENHANCED);
SIMPLE_TYPE_DEFINITION("ESChar", ESChar, "iopc", "iopc::ESChar", IOPC_TYPE_ESCHAR, IOPC_TYPECLASS_ENHANCED);
SIMPLE_TYPE_DEFINITION("EShort", EShort, "iopc", "iopc::EShort", IOPC_TYPE_ESHORT, IOPC_TYPECLASS_ENHANCED);
SIMPLE_TYPE_DEFINITION("EUShort", EUShort, "iopc", "iopc::EUShort", IOPC_TYPE_EUSHORT, IOPC_TYPECLASS_ENHANCED);
SIMPLE_TYPE_DEFINITION("ELong", ELong, "iopc", "iopc::ELong", IOPC_TYPE_ELONG, IOPC_TYPECLASS_ENHANCED);
SIMPLE_TYPE_DEFINITION("EULong", EULong, "iopc", "iopc::EULong", IOPC_TYPE_EULONG, IOPC_TYPECLASS_ENHANCED);
SIMPLE_TYPE_DEFINITION("EFloat", EFloat, "iopc", "iopc::EFloat", IOPC_TYPE_EFLOAT, IOPC_TYPECLASS_ENHANCED);
SIMPLE_TYPE_DEFINITION("EDouble", EDouble, "iopc", "iopc::EDouble", IOPC_TYPE_EDOUBLE, IOPC_TYPECLASS_ENHANCED);
SIMPLE_TYPE_DEFINITION("ELDouble", ELDouble, "iopc", "iopc::ELDouble", IOPC_TYPE_ELDOUBLE, IOPC_TYPECLASS_ENHANCED);
SIMPLE_TYPE_DEFINITION("EWCharT", EWCharT, "iopc", "iopc::EWCharT", IOPC_TYPE_EWCHART, IOPC_TYPECLASS_ENHANCED);
SIMPLE_TYPE_DEFINITION("EBool", EBool, "iopc", "iopc::EBool", IOPC_TYPE_EBOOL, IOPC_TYPECLASS_ENHANCED);



}

#endif /*ENHANCEDNUMBER_H_*/
