#ifndef ENHANCEDTYPEBASE_H_
#define ENHANCEDTYPEBASE_H_

#include "iopccommon/defs.h"
#include "object.h"

namespace iopc {
	
	
class DLL_EXTERNAL EnhancedTypeBase : public Object {
private:
	bool dirty;
	bool* dirtyPtr;
	bool null;
public:
	virtual ~EnhancedTypeBase() {};

	EnhancedTypeBase() : dirty(false), dirtyPtr(NULL),null(true) {}
	
	void setDirtyPtr(bool* newDirtyPtr);
	
	virtual void setDirty(bool dirty = true);
	virtual inline bool isDirty() const { return dirty; }
	virtual inline void resetDirty() { setDirty(false); }
	
	virtual inline void setNull(bool null = true) {this->null = null; }
	virtual inline bool isNull() const { return null; }
	virtual inline void resetNull() { setNull(false); }
	
	/**
	 * Creates a copy of this object's value and returns a pointer to it.
	 * 
	 * Type of the copy is the type of the wrapped value (numbers) or char/wchar_t* for EString
	 * or EWString.
	 * 
	 * @param maxlength Maximum allocated length of externalised copy. For numbers this value is ignored. For strings this value must by greater than 0.
	 */
	virtual void* externalizeCopy(unsigned int maxlength = 0) = 0;
	
	/**
	 * Creates an empty buffer for obtaining the value.
	 *
	 * @param length Allocated length of externalised copy. For numbers this value is ignored. For strings this value must by greater than 0.
	 */
	virtual void* externalizeEmptyBuffer(unsigned int length = 0) = 0;
	
	/**
	 * Updates this object's value from the externalized buffer
	 */
	virtual void updateFromExternalized() = 0;
	
	virtual unsigned int getBufferSize() = 0;
	
	
	virtual const Type& getInnerType() const = 0;
	
//	virtual Object* clone() const = 0;

	static void init();
	
	
	/**
	 * Returns inner type for enhanced type.
	 * 
	 * Static variant for enhancedType.getInnerType()
	 */
	static const Type& getInnerTypeFor(const Type& enhancedType);
};
	
	
}

#endif /*ENHANCEDTYPEBASE_H_*/
