#ifndef TESTOBJECT_H_
#define TESTOBJECT_H_

#include "iopccommon/trace.h"
#include "iopcmeta/enhancedNumber.h"
#include "iopcmeta/enhancedString.h"
#include "iopcmeta/type.h"
#include "iopclib/databaseObject.h"
#include "iopclib/oidObject.h"
#include "iopclib/dbPtr.h"
#include <sstream>
#include <string>

class TestObject : public iopc::DatabaseObject
{
public:
	iopc::EInt attrInt;
	iopc::EDouble attrDouble;
	iopc::EString attrString;

	static void iopcInit(iopc::Type& t) {
		//iopc::LogWriter::logInfo("static IOPC_INIT_TEST_OBJECT");
		//t["db.mapping.oidmanaged"] = false;

		t.getAttribute("attrInt")["db.primaryKey"].setBoolValue(true);
		t.getAttribute("attrDouble")["db.index"].setStringValue("idxdbl");
		t.getAttribute("attrString")["db.primaryKey"].setBoolValue(true);
	}

public:

	void test() {
		iopc::LogWriter::logInfo("TestObject::test(): DbPtrTest");
		attrInt = 3;
		attrDouble = 2.6;
		attrString = "Ahoj lidi";
	}

	virtual std::string toString()
	{
		std::ostringstream s;
		s << "attrInt: " << attrInt << std::endl;
		s << "attrDouble: " << attrDouble << std::endl;
		s << "attrString: " << attrString;
		return s.str();
	}

	friend class iopc::TypeDesc<TestObject>;
};

class InheritedTestObject : public TestObject
{
public:
	iopc::EInt attrInt2;

	static void iopcInit(iopc::Type& t) {
		//t["db.mapping.oidmanaged"] = false;
		//t["db.table"] = "inheritedfaketypetable";
		//t.getAttribute("attrInt2")["db.column"] = "colInt2";
		// keys and oidmanaged are derived..
	}

	virtual std::string toString()
	{
		std::ostringstream s;
		s << "attrInt: " << attrInt << std::endl;
		s << "attrDouble: " << attrDouble << std::endl;
		s << "attrString: " << attrString << std::endl;
		s << "attrInt2: " << attrInt2;
		return s.str();
	}

	friend class iopc::TypeDesc<InheritedTestObject>;
};

class InheritedTestObject2 : public InheritedTestObject
{
public:
	iopc::EInt attrInt3;

	static void iopcInit(iopc::Type& t) {
		//t["db.mapping.oidmanaged"] = false;
		t["db.mapping.type"] = "horizontal";
		t.getAttribute("attrInt3")["db.index.unique"].setStringValue("idxint3");
		//t["db.table"] = "inheritedfaketypetable";
		//t.getAttribute("attrInt2")["db.column"] = "colInt2";
		// keys and oidmanaged are derived..
	}

	virtual std::string toString()
	{
		std::ostringstream s;
		s << "attrInt: " << attrInt << std::endl;
		s << "attrDouble: " << attrDouble << std::endl;
		s << "attrString: " << attrString << std::endl;
		s << "attrInt2: " << attrInt2 << std::endl;
		s << "attrInt3: " << attrInt3;
		return s.str();
	}

	friend class iopc::TypeDesc<InheritedTestObject2>;
};

class UserTable : public iopc::DatabaseObject
{
	public:
	iopc::EString tableName;
	iopc::EInt blocks;

	static void iopcInit(iopc::Type& t) {
		//t["scripts.exclude"].setBoolValue(true);
		t["db.transient"].setBoolValue(true);
		//t["db.table"].setStringValue("user_tables");
		t.getAttribute("tableName")["db.column"].setStringValue("table_name");
		//t.getAttribute("tableName")["db.primaryKey"].setBoolValue(true);
		t.getAttribute("tableName")["db.type.length"] = 30;
		t.getAttribute("blocks")["db.column"].setStringValue("blocks");
	}

	friend class iopc::TypeDesc<UserTable>;
};

class Person : public iopc::OidObject {
public:
	iopc::EString surname;
public:

	static void iopcInit(iopc::Type& t) {
		t.getAttribute("surname")["db.type.length"] = 50;
		//t["db.mapping.type"] = "object";
	}

	friend class iopc::TypeDesc<Person>;
};

class Employee : public Person {
public:
	iopc::EBool isHappy;
public:

	static void iopcInit(iopc::Type& t) {
		//t["db.mapping.type"] = "object";
	}

	friend class iopc::TypeDesc<Employee>;
};

class Company : public iopc::OidObject {
public:
	iopc::EString name;
	iopc::DbPtr<Person> owner;
public:

	static void iopcInit(iopc::Type& t) {
		t.getAttribute("name")["db.type.length"] = 50;
		//t["db.mapping.type"] = "object";
	}

	friend class iopc::TypeDesc<Company>;
};


#endif /*TESTOBJECT_H_*/
