#include "tests/cacheKeeperTest.h"
#include "iopccommon/trace.h"
#include "iopcdb/driverManager.h"
#include "iopclib/db/cachedDatabase.h"
#include "iopclib/db/cachedConnection.h"
#include "testSchema/diamond.h"
#include "iopclib/cache/composedCache.h"
#include "iopclib/cache/cacheKeeper.h"
#include "iopclib/cache/voidCache.h"
#include "iopclib/cache/lruCache.h"
#include "iopclib/cache/arcCache.h"
#include "iopclib/dbPtr.h"
#include <vector>

//#include "testSchema/fakeObject.h"
//#include "testSchema/fakeType.h"

using namespace iopc;
using namespace std;

CacheKeeperTest::CacheKeeperTest(const std::string& testName)
 : TestCase(testName)
{

}

CacheKeeperTest::~CacheKeeperTest()
{
}


bool CacheKeeperTest::doTest()
{
	Database* db = NULL;
	CachedConnection* conn = NULL;
	bool ret = true;
	try {
		db = new CachedDatabase(DriverManager::getDriver("IopcOracle10g").getDatabase(DBNAME), new ComposedCache());
		conn = (CachedConnection*)db->getConnection(CONNSTR);
		conn->open();
		conn->setAutoCommit(false);

		CacheKeeper cacheKeeper(-15000, 20, 40);
		cacheKeeper.setCache(conn, new HashArcCache());
		cacheKeeper.setStrategy(conn, Strategy::xAccess);
		/*cacheKeeper.SetStrategy(*C2, Class["PersistentObject"], Strategy::async);
		cacheKeeper.SetStrategy(*C2, Class["Person"], Strategy::async);
		cacheKeeper.SetStrategy(*C2, Class["Leader"], Strategy::xAccess);*/
		cacheKeeper.runMaintainer();

		logMsg("Writing 50 objects");

		vector<OidType> oids;

		for (int i = 1; i <= 50; i++) {
			if ((i % 10) == 0) {
				logMsg("i: ", i);
			}
			DbPtr<DiamondClass4> objectPtr2;
			objectPtr2.bePersistent(conn);
			oids.push_back(objectPtr2.getPersId().getOid());
		}
		logMsg("Commit");
		conn->commit();

		logMsg("Updating 50 objects");
		for (vector<OidType>::iterator it = oids.begin(); it != oids.end(); it++) {
			DbPtr<DiamondClass4> loaded(conn, *it);
			loaded->test4 = "xxtest";
		}
		logMsg("Commit");
		conn->commit();
		logMsg("Deleting 50 objects, half only from cache");
		int j=1;
		for (vector<OidType>::iterator it = oids.begin(); it != oids.end(); it++) {
			DbPtr<DiamondClass4> loaded(conn, *it);
			if ((j % 2) == 0) {
				loaded.dbDelete();
			} else {
				loaded.cacheDelete();
			}
			j++;
		}
		conn->commit();

		cacheKeeper.stopMaintainer();
	} catch (IopcException& e) {
		LogWriter::logError(e.what());
		ret = false;
	}

	if (db && conn) db->returnConnection(conn);
	if (db) DriverManager::getDriver("IopcOracle10g").returnDatabase(db);

	return ret;
}

//static CacheKeeperTest dbPtrTestInstance("CacheKeeper Test");
